<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * @category    Pinpoint
 * @copyright   Copyright (c) 2020 Pinpoint Designs. (http://www.pinpointdesigns.co.uk)
 * @author      Daniel Cuevas <daniel@pinpointdesigns.co.uk>
 */

namespace Pinpoint\DropShipping\Controller\Adminhtml\Dropship;

use Magento\Backend\App\Action;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Message\ManagerInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;
use Magento\Sales\Model\OrderRepository;
use Pinpoint\DropShipping\Model\DropShipItemFactory;
use Pinpoint\DropShipping\Model\DropShipItemRepository;
use Pinpoint\DropShipping\Model\SupplierRepository;
use Magento\Framework\Event\ManagerInterface as EventManager;

/**
 * Class MassDropship
 */
class MassDropship extends Action
{
    const SUCCESSFUL_PAYMENT = 'successful_payment';
    const SUCCESSFUL_PAYMENT_PAYPAL = 'processing';
    const SUCCESSFUL_PAYMENT_BANK = 'successful_payment_bank';
    const SUCCESSFUL_PAYMENT_KLARNA = 'successful_payment_klarna';
    const ORDER_WITH_SUPPLIER = 'ordered_with_supplier';

    /**
     * @var OrderItemRepositoryInterface
     */
    private $orderItemRepository;
    /**
     * @var DropShipItemFactory
     */
    private $dropShipItemFactory;
    /**
     * @var OrderRepository
     */
    private $orderRepository;
    /**
     * @var DropShipItemRepository
     */
    private $dropShipItemRepository;
    /**
     * @var SupplierRepository
     */
    private $supplierRepository;
    /**
     * @var \Magento\Sales\Api\Data\OrderItemInterface[]
     */
    private $_items;
    /**
     * @var EventManager
     */
    protected $eventManager;

    /**
     * MassDropship constructor.
     * @param OrderItemRepositoryInterface $orderItemRepository
     * @param DropShipItemFactory $dropShipItemFactory
     * @param OrderRepository $orderRepository
     * @param DropShipItemRepository $dropShipItemRepository
     * @param SupplierRepository $supplierRepository
     * @param EventManager $eventManager
     * @param Action\Context $context
     */
    public function __construct(
        OrderItemRepositoryInterface $orderItemRepository,
        DropShipItemFactory $dropShipItemFactory,
        OrderRepository $orderRepository,
        DropShipItemRepository $dropShipItemRepository,
        SupplierRepository $supplierRepository,
        EventManager $eventManager,
        Action\Context $context
    )
    {
        parent::__construct($context);
        $this->orderItemRepository = $orderItemRepository;
        $this->dropShipItemFactory = $dropShipItemFactory;
        $this->orderRepository = $orderRepository;
        $this->dropShipItemRepository = $dropShipItemRepository;
        $this->supplierRepository = $supplierRepository;
        $this->eventManager = $eventManager;
    }

    /**
     * @inheritDoc
     */
    public function execute()
    {
        try {

            $orderId = $this->getRequest()->getParam('order_id');
            $itemIds = $this->getRequest()->getParam('item_id');
            $supplierIds = $this->getRequest()->getParam('supplier');

            $order = $this->orderRepository->get($orderId);
            $allowedStatuses = [
                self::SUCCESSFUL_PAYMENT,
                self::SUCCESSFUL_PAYMENT_PAYPAL,
                self::SUCCESSFUL_PAYMENT_BANK,
                self::SUCCESSFUL_PAYMENT_KLARNA
            ];
            if (!in_array($order->getStatus(), $allowedStatuses)) {
                throw new LocalizedException(__('The order status must in one of the "Successful Payment" statuses'));
            }

            $this->initializeOrderItems($order);

            $count = 0;

            $dropShippedItems = [];
            $dropShippedItemIds = [];
            
            foreach ($itemIds as $key => $itemId) {
                if (isset($supplierIds[$key])) {
                    if(!$supplierIds[$key])
                        continue;
                    
                    $supplier = $this->supplierRepository->get($supplierIds[$key]);

                    if (is_null($supplier->getMethod())) {
                        throw new LocalizedException(__('The supplier export method is not set, so the order item cannot be dropshipped.'));
                    }

                    $orderItem = $this->getOrderItem($itemId);
                    $dropshipItem = $this->dropShipItemFactory->create();
                    $dropshipItem->setDropshippingData($orderItem, $supplier);

                    $dropshipItem->save();
                    $dropShippedItems[] = $dropshipItem;
                    $dropShippedItemIds[] = $dropshipItem->getId();
                    $count++;
                }
            }

            if ($count) {
                $this->messageManager->addSuccessMessage("Dropshipped {$count} item(s).");

                if ($count === count($itemIds)) {
                    $order->setStatus(self::ORDER_WITH_SUPPLIER);
                    $this->messageManager->addSuccessMessage('The order status is now "Preparing for Despatch"');
                }

                $this->orderRepository->save($order);

                $this->_objectManager->get(\Magento\Backend\Model\Session::class)->setJustDropshippedItems($dropShippedItemIds);

                $this->eventManager->dispatch('dropshipping_order_dropship', [
                    'order' => $order,
                    'drop_shipped_items' => $dropShippedItems
                ]);
            }

        } catch (LocalizedException $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage('There was an error when trying to complete the dropshipping process. Please try again.');
        }

       return $this->_redirect('sales/order/view', ['order_id' => $orderId]);
    }

    private function getOrderItem($itemId)
    {
        if (isset($this->_items[$itemId])) {
            return $this->_items[$itemId];
        }

        return null;
    }

    private function initializeOrderItems(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        if (is_null($this->_items)) {
            $this->_items = [];

            foreach ($order->getItems() as $item) {
                $this->_items[$item->getItemId()] = $item;
            }
        }
    }
}
